# Copyright (c) 2013 Cloudera, Inc. All rights reserved.

import errno
import logging
import os
import subprocess
import tempfile
import threading
import time

KINIT_BIN = os.environ.get('KINIT_BIN', 'kinit')
LOG = logging.getLogger(__name__)

#
# A pathological KDC can renew every 10 minutes. But we don't
# want to renew on the dot. So give it some slack if it's almost
# at the limit.
#
_DEFAULT_RENEWAL_PERIOD = 10 * 60       # 10 minutes
_RENEWAL_SLACK = 10                     # 10 sec
_RENEWAL_PERIOD = float(os.environ.get('KRB_RENEWAL_PERIOD', _DEFAULT_RENEWAL_PERIOD))

_cred_manager = None
_cred_manager_lock = threading.Lock()        # protects singleton creation

os.environ['KRB5CCNAME'] = os.path.join(tempfile.gettempdir(), 'krb5cc_cm_agent')

class CredentialManager(threading.Thread):
  """
  A thread to periodically kinit to make sure that the agent has a valid ticket.
  (It doesn't matter who the principal is.) There are two challenges to
  consider.

  First, a valid (keytab, principal) combo could become invalid at any time. The
  CM user could've done a regenerate credential. When this happens, we want to
  throw away this cred and use something else.

  Second, the kinit could fail on a valid (keytab, principal) combo due to
  transient errors, like KDC going offline. We shouldn't throw away that valid
  credential.

  So we keep a stack of candidate creds. When a Process activates and attempt to
  kinit, we steal and stash its creds (stored in `_candidates'). The thread main
  loop periodically tries to reinit using the first candidate. On failure, it'll
  move to the next one. And so on. Special logic/hack is added to detect whether
  it's a KDC offline/transient failure.
  """

  def __init__(self):
    threading.Thread.__init__(self, name='CredentialManager')
    self.setDaemon(True)

    self._last_renewal = 0
    self._should_stop = False

    # A stack of (keytab, princ) tuples that could potentially work
    self._candidates = [ ]
    self._lock = threading.Lock()
    self._trigger = threading.Condition(self._lock)


  def async_kinit(self, keytab, princ):
    """
    Asynchronously attempt a kinit using the given credentials. If we already
    have a ticket belonging to a different principal, then this method may
    return without doing anything.

    This method checks for the last renewal time to prevent frequent kinits.
    """
    self._lock.acquire()
    try:
      # Push onto the stack and trigger
      self._candidates.append((keytab, princ))
      self._trigger.notify()
    finally:
      self._lock.release()


  def _run_once(self):
    """
    Pick the first credential candidate and try to kinit.
    If it succeeds, then put the cred back at the top of the candidate stack.

    Assumptions:
    - Only called from the manager thread. (Not thread safe).
    - The object lock has been acquired. (Will release lock around the
      actual kinit call.)
    - _candidates stack is not empty.

    Returns:
      0:          no error (success or no need to kinit yet)
      EAGAIN:     failed due to transient error (e.g. network error)
      EINVAL:     bad keytab or principal
    """
    # Don't kinit too often.
    now = time.time()
    if now - self._last_renewal < _RENEWAL_PERIOD - _RENEWAL_SLACK:
      return 0

    # Do kinit. Remember to release and reacquire the lock.
    keytab, princ = self._candidates.pop()
    self._lock.release()
    try:
      res = _do_actual_kinit(keytab, princ)
    finally:
      self._lock.acquire()

    # Update the renewal time on success
    if res == 0:
      self._last_renewal = now

    if res in (0, errno.EAGAIN):
      # The creds are not necessarily bad. Put it back.
      self._candidates.append((keytab, princ))

    return res


  def run(self):
    self._lock.acquire()
    while not self._should_stop:
      try:
        if LOG.isEnabledFor(logging.DEBUG):
          LOG.debug("Running with candidates: \n  %s" %
                    ("\n  ".join([ str(c) for c in self._candidates ])))
        # Try through all candidates, and retry if one is invalid
        while len(self._candidates) != 0:
          res = self._run_once()
          if res in (0, errno.EAGAIN):
            break

        # Sleep for the renewal period
        self._trigger.wait(_RENEWAL_PERIOD)

      except Exception, ex:
        LOG.exception(
          "Caught unexpected error. Will try to recover. %s" % (ex,))

  def stop(self):
    self._lock.acquire()
    try:
      self._should_stop = True
      self._trigger.notify()
    finally:
      self._lock.release()


  def _async_kinit_multi_for_test(self, *credentials):
    """
    credentials is a bunch of (keytab, princ) tuples. If the order is
    c1, c2, c3, then the manager will try them in that given order.
    """
    self._lock.acquire()
    try:
      # Push onto the stack and trigger
      cred_list = list(credentials)
      cred_list.reverse()
      for keytab, princ in cred_list:
        self._candidates.append((keytab, princ))
      self._trigger.notify()
    finally:
      self._lock.release()



def async_kinit_from_keytab(keytab_path, principal):
  """
  Ensure that the agent has a valid kerberos ticket (of any principal).
  If we already have a ticket belonging to a different principal, then this
  may be a no-op.
  """
  _get_cred_manager().async_kinit(keytab_path, principal)


def _get_cred_manager():
  global _cred_manager
  if _cred_manager is not None:
    return _cred_manager

  _cred_manager_lock.acquire()
  try:
    if _cred_manager is None:
      cmgr = CredentialManager()
      cmgr.start()
      _cred_manager = cmgr
  finally:
    _cred_manager_lock.release()

  return _cred_manager


def stop_cred_manager():
  """Used for testing"""
  global _cred_manager
  _cred_manager_lock.acquire()
  try:
    if _cred_manager is not None:
      _cred_manager.stop()
    _cred_manager = None
    return True
  finally:
    _cred_manager_lock.release()

  return False


def _do_actual_kinit(keytab_path, principal):
  """
  Do the actual kinit work. Returns:
    0:          success
    EAGAIN:     retry later, possible network error
    EINVAL:     bad keytab or principal
  """
  cmdv = [ KINIT_BIN,
          '-k',
          '-t', str(keytab_path),
          str(principal) ]
  LOG.debug("kinit as '%s' using %s" % (principal, keytab_path))
  popen = subprocess.Popen(cmdv,
                           stdout=subprocess.PIPE,
                           stderr=subprocess.STDOUT)
  stdout = popen.communicate()[0]
  if popen.returncode != 0:
    LOG.warn("Couldn't kinit as '%s' using %s --- %s" %
             (principal, keytab_path, stdout))

    # Error messages when KDC is unreachable
    if "Cannot resolve" in stdout or "Cannot contact" in stdout:
      return errno.EAGAIN
    return errno.EINVAL

  return 0
